<?php
namespace App;

use DI\ContainerBuilder;
use Interop\Container\ContainerInterface;

class App extends \DI\Bridge\Slim\App {

    protected function configureContainer(ContainerBuilder $builder)
    {
        $definitions = [
            'settings.displayErrorDetails' => true,
            
            'errorHandler' => function ($c) {
                return function ($request, $response, $e) use ($c) {
                    return $response->withStatus(500)->withJson([
                        'error'   => true,
                        'message' => $e->getMessage(),
                        'result'  => []
                    ]);
                };
            },

            \Doctrine\DBAL\Connection::class => function () {
                
                global $ENVIRONMENT_FILE;

                $environmentFile = $ENVIRONMENT_FILE;
                $environment     = json_decode($environmentFile, true);

                $connection = null;

                $myPlatform = new \Doctrine\DBAL\Platforms\SQLServerPlatform();

                $dbPassword = $environment['password'];
                if(isset($environment['encryptedPassword']) && $environment['encryptedPassword']) {
                    $dbPassword = $this->decrypt($environment['password']);
                }

                $connection = \Doctrine\DBAL\DriverManager::getConnection(array(
                    'user'        => $environment['user'],
                    'password'    => $dbPassword,
                    'host'        => $environment['host'],
                    'dbname'      => $environment['dbname'],
                    'driver'      => 'sqlsrv',
                    'charset'     => 'UTF-8',
                    'platform'    => $myPlatform,
                ), $config = new \Doctrine\DBAL\Configuration);

                return $connection;
            }
        ];

        $builder->addDefinitions($definitions);
    }

    const IV_SIZE = 16;
    const TEXT_SIZE = 32;

    private static function padText($text, $size) {
        if (strlen($text) % $size) 
            return str_pad($text, strlen($text) + $size - strlen($text) % $size, "\0");
        return $text;
    }

    private static function decodeEncryptedText($text) {
        $text = base64_decode($text);
        $iv_pos = hexdec($text[0])+1;
        $iv = substr($text, $iv_pos, self::IV_SIZE);
        $text_arr = explode($iv, $text);
        $text_arr[0] = substr($text_arr[0], 1);
        $text = implode('', $text_arr);
        $iv = self::padText($iv, self::IV_SIZE);
        return array('text' => $text, 'iv' => $iv);
    }

    private static function decrypt($text, $salt = 'teknisa') {
        $cipher = "AES-256-CBC";
        $decoded = self::decodeEncryptedText($text);
        $text = openssl_decrypt($decoded['text'], $cipher, $salt, OPENSSL_NO_PADDING, $decoded['iv']);
        $text = trim($text);
        return $text;
    }
}
